@@ -326,13 +326,6 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
 @click.argument(
     "module", nargs=-1, required=False, shell_complete=_get_available_modules
 )
-@click.option(
-    "--since-git-sha",
-    "-i",
-    default=None,
-    is_flag=False,
-    help="Extracts modules changed since this git sha and updates them",
-)
 @click.option(
     "--installed-modules",
     "-i",
@@ -416,6 +409,13 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
     is_flag=True,
     help="dont check for outdated modules (for migrations suitable)",
 )
+@click.option(
+    "--since-git-sha",
+    "-G",
+    default=None,
+    is_flag=False,
+    help="Extracts modules changed since this git sha and updates them",
+)
 @pass_config
 @click.pass_context
 def update(
@@ -594,6 +594,8 @@ def update(
                             _technically_update(ex.affected_modules)
                         except Exception as ex:
                             raise UpdateException(module) from ex
+                        else:
+                            raise Exception("Error at update")
                     else:
                         raise UpdateException(module)
 
