@@ -11,7 +11,6 @@ import os
 import click
 from .odoo_config import current_version
 from .odoo_config import MANIFEST
-from .tools import _is_dirty
 from .odoo_config import customs_dir
 from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
@@ -37,10 +36,12 @@ def src(config):
 @click.pass_context
 @pass_config
 def ensure_odoosh_repo(config, ctx):
-    _ensure_odoosh_repo()
+    _ensure_odoosh_repo(ctx)
 
 
-def _ensure_odoosh_repo():
+def _ensure_odoosh_repo(ctx):
+    from gimera.gimera import apply as gimera_apply
+
     odoosh_path = Path(os.environ["ODOOSH_REPO"] or "../odoo.sh").resolve().absolute()
     if not odoosh_path.exists():
         subprocess.check_call(
@@ -51,13 +52,8 @@ def _ensure_odoosh_repo():
                 odoosh_path,
             ]
         )
-        subprocess.check_call(
-            [
-                "gimera",
-                "apply",
-            ],
-            cwd=odoosh_path.absolute(),
-        )
+        with cwd(odoosh_path.absolute()):
+            ctx.invoke(gimera_apply)
     return odoosh_path
 
 
@@ -101,8 +97,7 @@ def _build_gimera(path):
 
 
 def _turn_into_odoosh(ctx, path):
-
-    _ensure_odoosh_repo()
+    _ensure_odoosh_repo(ctx)
     content_changed = _build_gimera(path)
 
     repos = yaml.safe_load((path / "gimera.yml").read_text())
@@ -120,6 +115,7 @@ def _find_duplicate_modules():
 
 def _apply_gimera_if_required(ctx, path, content, force_do=False):
     from gimera.gimera import apply as gimera
+
     def needs_apply():
         for repo in content["repos"]:
             repo_path = path / repo["path"]
@@ -130,7 +126,9 @@ def _apply_gimera_if_required(ctx, path, content, force_do=False):
     if force_do or needs_apply():
         with cwd(path):
             ctx.invoke(gimera, recursive=True)
-            click.secho("Restarting reloading because gimera apply was done", fg="yellow")
+            click.secho(
+                "Restarting reloading because gimera apply was done", fg="yellow"
+            )
             Commands.invoke(ctx, "reload", no_auto_repo=True)
 
             from .module_tools import Modules
@@ -138,13 +136,14 @@ def _apply_gimera_if_required(ctx, path, content, force_do=False):
             modules = Modules()
             all_modules = modules.get_all_modules_installed_by_manifest()
 
+
 @src.command()
 @click.pass_context
 def apply_gimera_if_required(ctx):
     path = customs_dir()
     gimera_file = path / "gimera.yml"
     if not gimera_file.exists():
-        if MANIFEST().get('auto_repo', False):
+        if MANIFEST().get("auto_repo", False):
             _build_gimera(path)
         else:
             return
@@ -470,44 +469,29 @@ def _identify_duplicate_modules(check):
                     )
 
 
-@src.command
-@click.option("-f", "--fix-not-in-manifest", is_flag=True)
-@pass_config
-def show_installed_modules(config, fix_not_in_manifest):
-    from .module_tools import DBModules, Module
-    from .odoo_config import customs_dir
-
-    path = customs_dir()
-    collected = []
-    not_in_manifest = []
-    manifest = MANIFEST()
-    setinstall = manifest.get("install", [])
-
-    for module in sorted(DBModules.get_all_installed_modules()):
-        try:
-            mod = Module.get_by_name(module)
-            click.secho(f"{module}: {mod.path}", fg="green")
-            if not [x for x in setinstall if x == module]:
-                not_in_manifest.append(module)
-        except KeyError:
-            collected.append(module)
-
-    for module in not_in_manifest:
-        if fix_not_in_manifest:
-            setinstall += [module]
-            click.secho(f"Added to manifest: {module}", fg="green")
-        else:
-            click.secho(f"Not in MANIFEST: {module}", fg="yellow")
-    for module in collected:
-        click.secho(f"Not in filesystem: {module}", fg="red")
-
-    if fix_not_in_manifest:
-        manifest["install"] = setinstall
-        manifest.rewrite()
-
 
 @src.command(name="pretty-print-manifest")
 def pretty_print_manifest():
     from .odoo_config import MANIFEST
 
     MANIFEST().rewrite()
+
+
+@src.command()
+@pass_config
+@click.argument("module")
+def security(config, module, model):
+    from .module_tools import Modules, Module
+
+    modules = Modules()
+    module = modules.get_by_name(module)
+
+    def ensure_secfile():
+        header = "model_id:id,group_id:id,id,name,perm_read,perm_write,perm_create,perm_unlink"
+        filepath = module.path / "security" / "ir.model.access.csv"
+        filepath.parent.mkdir(exist_ok=True)
+        if not filepath.read_text():
+            filepath.write_text(header + "\n")
+
+    # give rights to choose
+    # TODO ...
