@@ -9,6 +9,7 @@ import pickle
 import os
 import shutil
 import uuid
+from gimera.repo import Repo
 from .tools import get_hash, get_git_hash
 from .tools import __try_to_set_owner as try_to_set_owner
 from .tools import measure_time
@@ -84,7 +85,6 @@ def exe(*params):
 
 
 def delete_qweb(config, modules):
-
     with get_conn_autoclose(config) as cr:
         if modules != "all":
             cr.execute("select name from ir_module_module where name = %s", (modules,))
@@ -336,7 +336,6 @@ def make_customs(ctx, path):
         abort("Path is not empty: {}".format(path))
 
     import inquirer
-    from git import Repo
     from .tools import copy_dir_contents
 
     dir = get_template_dir()
@@ -360,16 +359,16 @@ def make_customs(ctx, path):
 
     manifest_file = path / "MANIFEST"
     manifest = eval(manifest_file.read_text())
+    raise Exception("Rewrite to use gimera")
 
     click.echo("Checking for odoo repo at env variable 'ODOO_REPO'")
     if os.getenv("ODOO_REPO", ""):
         odoo_path = path / "odoo"
         repo_path = Path(os.environ["ODOO_REPO"])
         repo = Repo(repo_path)
-        repo.git.checkout(str(version))
+        repo.X("git", "checkout", str(version))
         odoo_path.mkdir()
-        sha = repo.head.object.hexsha
-        sha = repo.git.rev_parse(sha)
+        sha = repo.hex
         click.echo("Copying odoo with sha to local directory [{}]".format(sha))
         copy_dir_contents(repo_path, odoo_path, exclude=[".git"])
         manifest["odoo_commit"] = sha
@@ -643,7 +642,7 @@ class Modules(object):
         return Modules_Cache["modules"]
 
     @classmethod
-    @measure_time
+    # @profile
     def _get_modules(self, no_deptree=False):
         modnames = set()
         from .odoo_config import get_odoo_addons_paths
@@ -654,7 +653,13 @@ class Modules(object):
             Returns a list of full paths of all manifests
             """
             for path in reversed(get_odoo_addons_paths()):
-                for file in sorted(path.glob("*/" + manifest_file_names())):
+                mans = subprocess.check_output(
+                    ["find", ".", "-maxdepth", "2", "-name", manifest_file_names()],
+                    cwd=path,
+                    encoding="utf8",
+                ).splitlines()
+                for file in sorted(mans):
+                    file = path / file
                     modname = file.parent.name
                     if modname in modnames:
                         continue
@@ -663,7 +668,7 @@ class Modules(object):
 
         modules = {}
         all_manifests = get_all_manifests()
-        for m in all_manifests:
+        for m in list(all_manifests):
             module = Module(m)
             module.manifest_dict.get("just read manifest")
             modules[m.parent.name] = module
@@ -699,6 +704,7 @@ class Modules(object):
             else:
                 modules.append(module.name)
 
+    # @profile
     def get_customs_modules(self, mode=None):
         """
         Called by odoo update
@@ -816,7 +822,7 @@ class Modules(object):
                 auto_install_modules.append(module)
         return list(sorted(set(auto_install_modules)))
 
-    @measure_time
+    # @profile
     def get_filtered_auto_install_modules_based_on_module_list(self, module_list):
         def _transform_modulelist(module_list):
             for mod in module_list:
@@ -860,6 +866,7 @@ class Modules(object):
                 break
         return list(sorted(set(modules)))
 
+    # @profile
     def get_all_used_modules(self):
         """
         Returns all modules that are directly or indirectly
@@ -881,37 +888,33 @@ class Modules(object):
         return list(result)
 
     def get_all_external_dependencies(self, modules):
-        pydeps = []
-        deb_deps = []
+        global_data = {"pip": []}
         for module_name in modules:
             module = Module.get_by_name(module_name)
             if module.path is None:
                 raise Exception(f"Module has no path: {module_name}")
             file = module.path / "external_dependencies.txt"
             new_deps = []
+
+            def extract_deps(data):
+                global_data["pip"].extend(data.get("pip", data.get("python", [])))
+                for k, v in data.items():
+                    if k not in ["pip", "python"]:
+                        global_data.setdefault(k, []).extend(v)
+
             if file.exists():
                 try:
                     content = json.loads(file.read_text())
                 except Exception as e:
-                    click.secho(
-                        "Error parsing json in\n{}:\n{}".format(file, e), fg="red"
-                    )
+                    click.secho("Error parsing json in\n{file}:\n{e}", fg="red")
                     click.secho(file.read_text(), fg="red")
                     sys.exit(1)
-                new_deps = content.get("pip", [])
-                pydeps += new_deps
-                deb_deps += content.get("deb", [])
+                extract_deps(content)
             else:
-                new_deps = module.manifest_dict.get("external_dependencies", {}).get(
-                    "python", []
-                )
-                pydeps += new_deps
+                extract_deps(module.manifest_dict.get("external_dependencies", {}))
 
-            # if new_deps:
-            #    click.secho(f"Adding python dependencies {','.join(new_deps)} from {module.name}", fg='yellow')
-
-        pydeps = self.resolve_pydeps(pydeps)
-        return {"pip": pydeps, "deb": deb_deps}
+        global_data["pip"] = self.resolve_pydeps(set(global_data["pip"]))
+        return global_data
 
     def resolve_pydeps(self, pydeps):
         pydeps = list(set(pydeps))
@@ -1038,7 +1041,7 @@ class Module(object):
         self._manifest_path = None
         self._dep_tree = None
         if path:
-            self.__init_path(path)
+            self.__init_path(path, manifest_file_names())
             self.path = self._manifest_path.parent
         else:
             self.path = None
@@ -1052,31 +1055,32 @@ class Module(object):
     def exists(self):
         return bool(self.path)
 
-    def __init_path(self, path):
+    def __init_path(self, path, manifest_filename):
         path = Path(path)
+        _customs_dir = customs_dir()
 
         remember_cwd = os.getcwd()
         try:
-            cwd = Path(os.getcwd())
+            cwd = Path(remember_cwd)
             if str(path).startswith("/"):
                 try:
-                    path = path.relative_to(customs_dir())
-                    os.chdir(customs_dir())
-                except:
+                    path = path.relative_to(_customs_dir)
+                    os.chdir(_customs_dir)
+                except Exception:
                     try:
                         path = path.relative_to(cwd)
                     except ValueError:
-                        path = path.relative_to(customs_dir())
+                        path = path.relative_to(_customs_dir)
                         os.chdir(
-                            customs_dir()
+                            _customs_dir
                         )  # reset later; required that parents works
             p = path if path.is_dir() else path.parent
 
             for p in [p] + list(p.parents):
-                if (p / manifest_file_names()).exists():
+                if (p / manifest_filename).exists():
                     if ".git" in p.parts:
                         continue
-                    self._manifest_path = p / manifest_file_names()
+                    self._manifest_path = p / manifest_filename
                     break
             if not getattr(self, "_manifest_path", ""):
                 raise Module.IsNot((f"no module found for {path}"))
@@ -1100,9 +1104,11 @@ class Module(object):
                 )
                 self._manifest_dict = eval(content)  # TODO safe
 
-            except (SyntaxError, Exception):
-                click.secho((f"error at file: {self.manifest_path}"), fg="red")
-                raise
+            except (SyntaxError, Exception) as e:
+                click.secho(
+                    (f"error at file: {self.manifest_path}:\n{str(e)}"), fg="red"
+                )
+                sys.exit(-1)
         return self._manifest_dict
 
     def __make_path_relative(self, path):
@@ -1313,7 +1319,7 @@ class Module(object):
         for to_remove in doc.xpath("//template[1] | //template[xpath[not(*)]]"):
             to_remove.getparent().remove(to_remove)
 
-        if current_version() >= 15.0:
+        if current_version() > 14.0:
             manifest = self.path / "__manifest__.py"
             jsoncontent = eval(manifest.read_text())
             jsoncontent.setdefault("assets", {})
@@ -1404,9 +1410,9 @@ class Module(object):
 
         mod[DATA_NAME] = []
         mod["demo"] = []
-        if current_version() <= 14.0:
+        mod["qweb"] = []
+        if current_version() < 14.0:
             mod["css"] = []
-            mod["qweb"] = []
         is_web = False
 
         for local_path in all_files:
@@ -1420,9 +1426,16 @@ class Module(object):
                     is_web = True
                     if local_path.suffix == ".xml":
                         if "qweb" in mod:
-                            mod["qweb"].append(str(local_path))
+                            if str(local_path) not in mod["qweb"]:
+                                if current_version() <= 14.0:
+                                    mod["qweb"].append(str(local_path))
+                                else:
+                                    mod["qweb"].append(
+                                        self.name + "/" + str(local_path)
+                                    )
                 else:
-                    mod[DATA_NAME].append(str(local_path))
+                    if local_path.name not in ["gimera.yml"]:
+                        mod[DATA_NAME].append(str(local_path))
             elif local_path.suffix == ".js":
                 pass
             elif local_path.suffix in [".css", ".less", ".scss"]:
@@ -1440,6 +1453,15 @@ class Module(object):
         if "depends" in mod:
             mod["depends"].sort()
 
+        if current_version() > 14.0:
+            if "qweb" in mod:
+                mod.setdefault("assets", {})
+                mod["assets"].setdefault("web.assets_qweb", [])
+                mod["assets"]["web.assets_qweb"] += mod.pop("qweb")
+                mod["assets"]["web.assets_qweb"] = list(
+                    sorted(set(mod["assets"]["web.assets_qweb"]))
+                )
+
         # now sort again by inspecting file content - if __openerp__.sequence NUMBER is found, then
         # set this index; reason: some times there are wizards that reference forms and vice versa
         # but cannot find action ids
@@ -1470,6 +1492,10 @@ class Module(object):
         sorted_by_index = sorted(sorted_by_index, key=lambda x: x[0])
         mod[DATA_NAME] = [x[1] for x in sorted_by_index]
 
+        # remove assets.xml for newer versions
+        if current_version() > 14.0:
+            mod[DATA_NAME] = list(filter(lambda x: not x.endswith("/assets.xml"), mod[DATA_NAME]))
+
         if is_web:
             mod["web"] = True
         if "application" not in mod:
@@ -1482,6 +1508,17 @@ class Module(object):
             pp = pprint.PrettyPrinter(indent=4, stream=file)
             pp.pprint(data)
 
+    def calc_complexity(self):
+        """
+        Calculates the complexity of the module
+        """
+        res = {"loc": 0}
+        for file in self.get_all_files_of_module():
+            if file.suffix in [".py", ".csv", ".xml"]:
+                file = self.path / file
+                res["loc"] += len(file.read_text().splitlines())
+        return res
+
 
 def write_debug_instruction(instruction):
     (customs_dir() / ".debug").write_text(instruction)
