@@ -1,4 +1,4 @@
-import platform
+import uuid
 from subprocess import PIPE, STDOUT
 import hashlib
 import requests
@@ -6,6 +6,8 @@ import stat
 from contextlib import contextmanager
 import re
 import inquirer
+import cProfile
+import functools
 
 try:
     import arrow
@@ -65,14 +67,22 @@ class DBConnection(object):
     def get_psyco_connection(self, db=None):
         import psycopg2
 
-        conn = psycopg2.connect(
-            dbname=db or self.dbname,
-            user=self.user,
-            password=self.pwd,
-            host=self.host,
-            port=self.port or None,
-            connect_timeout=int(os.getenv("PSYCOPG_TIMEOUT", "3")),
-        )
+        while True:
+            try:
+                conn = psycopg2.connect(
+                    dbname=db or self.dbname,
+                    user=self.user,
+                    password=self.pwd,
+                    host=self.host,
+                    port=self.port or None,
+                    connect_timeout=int(os.getenv("PSYCOPG_TIMEOUT", "3")),
+                )
+                break
+            except psycopg2.OperationalError as ex:
+                if "database system is starting up" in str(ex):
+                    time.sleep(2)
+                else:
+                    raise
         return conn
 
     @contextmanager
@@ -231,23 +241,21 @@ def _exists_table(conn, table_name):
     return record[0]
 
 
-def docker_list_containers():
-    container_ids = (
-        subprocess.check_output(
-            [
-                "docker",
-                "ps",
-                "-a",
-                "-q",
-                "--no-trunc",
-                "--filter",
-                f"name=^/{config.PROJECT_NAME}_postgres$",
-            ],
-            encoding="utf8",
-        )
-        .strip()
-        .splitlines()
-    )
+def docker_list_containers(project_name, service_name, status_filter=None):
+    cmd = [
+        "docker",
+        "ps",
+        "-a",
+        "-q",
+        "--no-trunc",
+        "--filter",
+        f"name=^/{project_name}_{service_name}$",
+    ]
+    if status_filter:
+        cmd += ["--filter", f"status={status_filter}"]
+    container_ids = subprocess.check_output(cmd, encoding="utf8").strip().splitlines()
+    return container_ids
+
 
 def _wait_postgres(config, timeout=600):
     started = arrow.get()
@@ -270,13 +278,15 @@ def _wait_postgres(config, timeout=600):
             .splitlines()
         )
 
-        postgres_containers = []
+        import docker
 
+        client = docker.from_env()
+        postgres_containers = []
         for container_id in container_ids:
             if not container_id:
                 continue
             state = _docker_id_state(container_id)
-            if state == 'running':
+            if state == "running":
                 postgres_containers += [container_id]
 
         deadline = arrow.get().shift(seconds=timeout)
@@ -321,29 +331,31 @@ def _wait_postgres(config, timeout=600):
 def _docker_id_state(container_id):
     status = subprocess.check_output(
         [
-            'docker',
-            'container',
-            'ls',
-            '--format',
-            '{{.State}}',
-            '--filter',
-            f'id={container_id}'
+            "docker",
+            "container",
+            "ls",
+            "--format",
+            "{{.State}}",
+            "--filter",
+            f"id={container_id}",
         ],
-        encoding='utf8',
+        encoding="utf8",
     ).strip()
     return status
 
+
 def docker_kill_container(container_id, remove=False):
-    subprocess.check_call(['docker', 'kill', container_id])
+    subprocess.check_call(["docker", "kill", container_id])
     if remove:
-        subprocess.check_call(['docker', 'rm', container_id])
+        subprocess.check_call(["docker", "rm", container_id])
+
 
 def _is_container_running(config, machine_name):
     container_id = __dc_out(config, ["ps", "-q", machine_name]).strip()
     if container_id:
         status = _docker_id_state(container_id)
         if status:
-            return status == 'running'
+            return status == "running"
     return False
 
 
@@ -391,7 +403,7 @@ def _remove_postgres_connections(connection, sql_afterwards=""):
 
 def __rename_db_drop_target(conn, from_db, to_db):
     if to_db in ("postgres", "template1"):
-        raise Exception(f"Invalid: {to_db}")
+        raise Exception("Invalid: {}".format(to_db))
     _remove_postgres_connections(conn.clone(dbname=from_db))
     _remove_postgres_connections(conn.clone(dbname=to_db))
     _execute_sql(
@@ -544,9 +556,9 @@ def __rm_file_if_exists(path):
 def __rmtree(config, path):
     path = str(path)
     if not path or path == "/":
-        raise Exception(f"Not allowed: {path}")
+        raise Exception("Not allowed: {}".format(path))
     if not path.startswith("/"):
-        raise Exception(f"Not allowed: {path}")
+        raise Exception("Not allowed: {}".format(path))
     if config:
         if not any(
             path.startswith(config.dirs["odoo_home"] + x) for x in ["/tmp", "/run/"]
@@ -599,7 +611,7 @@ def __empty_dir(dir, user_out=False):
                 if user_out:
                     click.secho(f"Removing {x.absolute()}")
                 x.unlink()
-    except Exception:
+    except:
         click.secho(f"Could not delete: {dir}", fg="red")
         raise
 
@@ -680,7 +692,7 @@ def __splitcomma(param):
         return [x.strip() for x in param.split(",") if x.strip()]
     elif isinstance(param, (tuple, list)):
         return list(param)
-    raise NotImplementedError("not impl")
+    raise Exception("not impl")
 
 
 def __make_file_executable(filepath):
@@ -714,7 +726,7 @@ def __try_to_set_owner(UID, path, abort_if_failed=True, verbose=False):
         try:
             try:
                 subprocess.check_output(["chown", str(UID), line])
-            except Exception:
+            except:
                 try:
                     subprocess.check_output(["sudo", "chown", str(UID), line])
                 except Exception as ex:
@@ -723,7 +735,7 @@ def __try_to_set_owner(UID, path, abort_if_failed=True, verbose=False):
 
             try:
                 subprocess.check_output(["chgrp", str(primary_group), line])
-            except Exception:
+            except:
                 try:
                     subprocess.check_output(["sudo", "chgrp", str(primary_group), line])
                 except:
@@ -842,6 +854,7 @@ def remove_webassets(conn):
         f"delete from ir_attachment where name ilike '%.scss' {ignore_url_str}",
         f"delete from ir_attachment where name ilike 'web_icon_data' {ignore_url_str}",
         f"delete from ir_attachment where name ilike 'web_editor.summernote.%' {ignore_url_str}",
+        f"delete from ir_attachment where name ilike 'web.assets_backend_prod_only.js'",
     ]
     try:
         for query in queries:
@@ -983,42 +996,6 @@ def _get_host_ip():
         return conn[2]
 
 
-def _is_dirty(repo, check_submodule, assert_clean=False):
-    from git import Repo
-    from git import InvalidGitRepositoryError
-    from git import NoSuchPathError
-
-    def raise_error():
-        if assert_clean:
-            click.secho(
-                ("Dirty directory - please cleanup: {repo.working_dir}"),
-                bold=True,
-                fg="red",
-            )
-            sys.exit(42)
-
-    if repo.is_dirty() or repo.untracked_files:
-        raise_error()
-        return True
-    if check_submodule:
-        try:
-            repo.submodules
-        except AttributeError:
-            pass
-        else:
-            for submodule in repo.submodules:
-                try:
-                    sub_repo = Repo(submodule.path)
-                except InvalidGitRepositoryError:
-                    click.secho(f"Invalid Repo: {submodule}", bold=True, fg="red")
-                except NoSuchPathError:
-                    click.secho(f"Invalid Repo: {submodule}", bold=True, fg="red")
-                else:
-                    if _is_dirty(sub_repo, True, assert_clean=assert_clean):
-                        raise_error()
-                        return True
-    return False
-
 
 def __assure_gitignore(gitignore_file, content):
     p = Path(gitignore_file)
@@ -1244,8 +1221,6 @@ def get_filesystem_of_folder(path):
     lines = (
         subprocess.check_output([df, "-T", path], encoding="utf8").strip().splitlines()
     )
-    if not lines:
-        return None
     fstype = list(filter(bool, lines[1].replace("\t", " ").split(" ")))[1]
     return fstype
 
@@ -1419,6 +1394,8 @@ def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
     from .odoo_config import MANIFEST
     from .odoo_config import current_version
     from .cli import cli, pass_config, Commands
+    from gimera.gimera import add as gimera_add
+    from gimera.gimera import apply as gimera_apply
 
     state = DBModules.get_meta_data(modulename)
     if state["state"] == "installed":
@@ -1426,27 +1403,14 @@ def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
 
     path = Path("addons_wodoo") / modulename
     if not path.exists():
-        subprocess.check_call(
-            [
-                "gimera",
-                "add",
-                "-u",
-                repo_url,
-                "--path",
-                path,
-                "--branch",
-                str(current_version()),
-                "--type",
-                "integrated",
-            ]
-        )
-        subprocess.check_call(
-            [
-                "gimera",
-                "apply",
-                path,
-            ]
+        ctx.invoke(
+            gimera_add,
+            url=repo_url,
+            path=str(path),
+            branch=str(current_version()),
+            type="integrated",
         )
+        ctx.invoke(gimera_apply, repos=str(path))
 
     # if not yet there, then pack into "addons_framework"
     manifest = MANIFEST()
@@ -1508,6 +1472,7 @@ def _get_setting(conn, key):
     if rec:
         return rec[0]
 
+
 @contextmanager
 def cwd(path):
     remember = os.getcwd()
@@ -1515,4 +1480,30 @@ def cwd(path):
     try:
         yield
     finally:
-        os.chdir(remember)
\ No newline at end of file
+        os.chdir(remember)
+
+
+@contextmanager
+def atomic_write(file):
+    tempfile = file.parent / f"{file.name}.{uuid.uuid4()}"
+    yield tempfile
+    if file.exists():
+        file.unlink()
+    tempfile.rename(file)
+
+
+def bash_find(path, name=None, wholename=None, type=None):
+    cmd = [
+        "find",
+    ]
+    if type:
+        cmd += [
+            "-type",
+            type,
+        ]
+    if wholename:
+        cmd += ["-wholename", wholename]
+    if name:
+        cmd += ["-name", name]
+    files = subprocess.check_output(cmd, cwd=path, encoding="utf8").splitlines()
+    return list(map(lambda x: Path(path) / x, files))
